# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node
import pyxb.binding.datatypes as xs

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="rDateTime">
    <xs:restriction base="xs:dateTime">
      <xs:minInclusive value="2010-01-01T00:00:00Z"/>
      <xs:maxInclusive value="2011-01-01T00:00:00Z"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="rDate">
    <xs:restriction base="xs:date">
      <xs:minInclusive value="2010-01-01"/>
      <xs:maxInclusive value="2011-01-01"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="rGmonthDay">
    <xs:restriction base="xs:gMonthDay">
      <xs:minInclusive value="--01-01"/>
      <xs:maxInclusive value="--02-01"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="rTime">
    <xs:restriction base="xs:time">
      <xs:minInclusive value="00:00:00Z"/>
      <xs:maxInclusive value="01:00:00Z"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="rDecimal">
    <xs:restriction base="xs:decimal">
      <xs:minInclusive value="0"/>
      <xs:maxInclusive value="2340"/>
    </xs:restriction>
  </xs:simpleType>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0231 (unittest.TestCase):
    def testFacets (self):
        self.assertEqual('2010-01-01T00:00:00Z', rDateTime._CF_minInclusive.value().xsdLiteral())
        self.assertEqual('2010-01-01', rDate._CF_minInclusive.value().xsdLiteral())
        self.assertEqual('--01-01', rGmonthDay._CF_minInclusive.value().xsdLiteral())
        self.assertEqual('00:00:00Z', rTime._CF_minInclusive.value().xsdLiteral())
        self.assertEqual('2340.0', rDecimal._CF_maxInclusive.value().xsdLiteral())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xsd
import pyxb.utils.domutils
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-ctd-simple.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestCTDSimple (unittest.TestCase):

    def testClause4 (self):
        self.assertTrue(clause4._IsSimpleTypeContent())
        self.assertTrue(clause4._TypeDefinition == xsd.string)
        self.assertEqual(None, clause4._TypeDefinition._CF_length.value())

    def testClause3 (self):
        self.assertTrue(clause3._IsSimpleTypeContent())
        self.assertTrue(issubclass(clause3, clause4))
        self.assertTrue(clause3._TypeDefinition == xsd.string)

    def testClause2 (self):
        self.assertTrue(clause2._IsSimpleTypeContent())
        self.assertTrue(issubclass(clause2, ctype))
        self.assertTrue(issubclass(clause2._TypeDefinition, xsd.string))
        self.assertEqual(6, clause2._TypeDefinition._CF_length.value())

    def testClause1_1 (self):
        self.assertTrue(clause1_1._IsSimpleTypeContent())
        self.assertTrue(issubclass(clause1_1, clause4))
        self.assertTrue(issubclass(clause1_1._TypeDefinition, xsd.string))
        self.assertEqual(2, clause1_1._TypeDefinition._CF_minLength.value())
        self.assertEqual(4, clause1_1._TypeDefinition._CF_maxLength.value())

    def testClause1_2 (self):
        self.assertTrue(clause1_2._IsSimpleTypeContent())
        self.assertTrue(issubclass(clause1_2, clause4))
        self.assertTrue(issubclass(clause1_2._TypeDefinition, xsd.string))
        self.assertEqual(6, clause1_2._TypeDefinition._CF_length.value())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node
import pyxb.namespace
import xml.dom.minidom as minidom

import os.path
xst = '''<?xml version="1.0" encoding="utf-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="bigpos">
    <xs:restriction base="xs:integer">
      <xs:minInclusive value="9223372036854775808"/>
      <xs:maxInclusive value="9223372036854775810"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="bigneg">
    <xs:restriction base="xs:integer">
      <xs:minInclusive value="-9223372036854775810"/>
      <xs:maxInclusive value="-9223372036854775808"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:complexType name="multiples">
    <xs:sequence>
      <xs:element name="e" minOccurs="0" maxOccurs="32" type="bigpos"/>
    </xs:sequence>
  </xs:complexType>
</xs:schema>
'''


code = pyxb.binding.generate.GeneratePython(schema_text=xst)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

import copy

class TestIssue22 (unittest.TestCase):

    SMALL = (six.long_type(1) << 63)
    LARGE = (six.long_type(1) << 63) + 2

    def testBigPos (self):
        self.assertEqual(self.SMALL, bigpos._CF_minInclusive.value())
        self.assertEqual(self.LARGE, bigpos._CF_maxInclusive.value())

    def testBigNeg (self):
        self.assertEqual(-self.SMALL, bigneg._CF_maxInclusive.value())
        self.assertEqual(-self.LARGE, bigneg._CF_minInclusive.value())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node
import pyxb.binding.datatypes as xs

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="digits">
    <xs:restriction base="xs:byte">
      <xs:minInclusive value="1"/>
      <xs:maxInclusive value="3"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="english">
    <xs:restriction base="xs:string">
      <xs:enumeration value="one"/>
      <xs:enumeration value="two"/>
      <xs:enumeration value="three"/>
      <xs:enumeration value="b@d"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="welsh">
    <xs:restriction base="xs:string">
      <xs:enumeration value="un"/>
      <xs:enumeration value="dau"/>
      <xs:enumeration value="tri"/>
      <xs:enumeration value="b/d"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="tUnion">
    <xs:union memberTypes="digits english welsh">
      <xs:simpleType>
        <xs:restriction base="xs:string">
          <xs:enumeration value="ichi"/>
          <xs:enumeration value="ni"/>
          <xs:enumeration value="san"/>
          <xs:enumeration value="b?d"/>
        </xs:restriction>
      </xs:simpleType>
    </xs:union>
  </xs:simpleType>
  <xs:element name="union" type="tUnion"/>
  <xs:simpleType name="bad">
    <xs:restriction base="xs:string">
      <xs:enumeration value="b@d"/>
      <xs:enumeration value="b/d"/>
      <xs:enumeration value="b?d"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:simpleType name="moreBad">
    <xs:union memberTypes="bad tUnion"/>
  </xs:simpleType>

</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

def SET_lu (instance, v):
    instance.lu = v

class TestTrac0038 (unittest.TestCase):
    """Visibility of enumerations within unions"""
    def testBasic (self):
        d = union('1')
        self.assertTrue(isinstance(d, digits))
        e = union('one')
        self.assertTrue(isinstance(e, english))
        self.assertEqual(e, english.one)
        self.assertEqual(e, tUnion.one)
        w = union('dau')
        self.assertTrue(isinstance(w, welsh))
        self.assertEqual(w, welsh.dau)
        self.assertEqual(w, tUnion.dau)
        self.assertRaises(pyxb.SimpleTypeValueError, union, 'deux')
        n = union('ni')
        self.assertEqual(n, tUnion.ni)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
<xs:complexType name="tAddress">
  <xs:choice>
    <xs:sequence>
      <xs:element name="Line1" type="xs:string"/>
      <xs:element name="Line2" type="xs:string"/>
    </xs:sequence>
    <xs:sequence>
      <xs:element name="Missing" type="xs:string"/>
    </xs:sequence>
  </xs:choice>
</xs:complexType>
<xs:complexType name="tOther">
  <xs:sequence>
    <xs:element name="Header" type="xs:string"/>
    <xs:choice>
      <xs:sequence>
        <xs:element name="Special" type="tAddress"/>
        <xs:element name="Common" type="tAddress" minOccurs="0"/>
      </xs:sequence>
      <xs:sequence>
        <xs:element name="Common" type="tAddress"/>
      </xs:sequence>
    </xs:choice>
  </xs:sequence>
</xs:complexType>
<xs:element name="elt" type="tOther"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0033a (unittest.TestCase):
    def test (self):
        xml = '<elt><Header/><Common><Line1/><Line2/></Common></elt>'
        instance = CreateFromDocument(xml)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
           xmlns="http://schema.omg.org/spec/CTS2/1.0/Core"
           targetNamespace="http://schema.omg.org/spec/CTS2/1.0/Core"
           elementFormDefault="qualified">
    <xs:complexType mixed="true" name="tsAnyType">
        <xs:sequence>
            <xs:any maxOccurs="unbounded" minOccurs="0" namespace="##any" processContents="lax"/>
        </xs:sequence>
    </xs:complexType>

    <xs:element name="OpaqueData">
        <xs:complexType>
            <xs:sequence>
                <xs:element name="v" type="tsAnyType" minOccurs="1"/>
            </xs:sequence>
        </xs:complexType>
    </xs:element>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest
# Note: Without this, there's an exception thrown below because the
# binding for the namespace isn't available; PyXB falls back to a raw
# DOM model there.  With this, though, you run afoul of trac/153 which
# may not be fixed in the PyXB 1.1.x series.
#import pyxb.bundles.common.xhtml1

class TestTrac0155 (unittest.TestCase):
    testxml = """<?xml version="1.0" encoding="UTF-8"?>
<OpaqueData xmlns="http://schema.omg.org/spec/CTS2/1.0/Core"
    xmlns:core="http://schema.omg.org/spec/CTS2/1.0/Core"
    xmlns:html="http://www.w3.org/1999/xhtml"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
    <core:v xmlns="http://www.w3.org/1999/xhtml"><ul><li>entry1</li><li>entry2</li></ul></core:v>
</OpaqueData> """

    def setUp (self):
        pyxb.utils.domutils.BindingDOMSupport.SetDefaultNamespace(Namespace.uri())

    def tearDown (self):
        pyxb.utils.domutils.BindingDOMSupport.SetDefaultNamespace(None)

    Expectedd = """<?xml version="1.0" encoding="utf-8"?><OpaqueData xmlns="http://schema.omg.org/spec/CTS2/1.0/Core" xmlns:ns1="http://www.w3.org/1999/xhtml"><v><ns1:ul><ns1:li>entry1</ns1:li><ns1:li>entry2</ns1:li></ns1:ul></v></OpaqueData>""".encode('utf-8')

    def test (self):
        txml = CreateFromDocument(self.testxml)
        #dom = txml.toDOM()
        #print dom.toprettyxml()
        #print txml.toxml()
        #print self.Expected
        self.assertEqual(txml.toxml('utf-8'), self.Expectedd)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
import pyxb
from pyxb.namespace import ExpandedName
import xml.dom
from pyxb.namespace import XMLSchema as xsd
xsd.validateComponentModel()
import pyxb.binding.datatypes as xsd_module

class TestExpandedName (unittest.TestCase):
    def testEquivalence (self):
        an1 = ExpandedName(None, 'string')
        en1 = ExpandedName(xsd, 'string')
        en2 = ExpandedName(xsd, 'string')
        en3 = ExpandedName(xsd, 'notString')
        self.assertEqual(en1, en2)
        self.assertEqual(en1, ( en1.namespace(), en1.localName() ))
        self.assertTrue(en1 == en2)
        self.assertFalse(en1 == en3)
        self.assertTrue(en1 != en3)
        self.assertTrue(an1 == an1.localName())
        self.assertFalse(an1 == en3.localName())
        self.assertTrue(an1 != en3.localName())
        self.assertFalse(an1 != an1.localName())

    def testRichComparisons (self):
        s1 = 'alpha'
        s2 = 'beta'
        an1 = ExpandedName(None, s1)
        en1 = ExpandedName(xsd, s1)
        an2 = ExpandedName(None, s2)
        en2 = ExpandedName(xsd, s2)
        # an1 ? an2
        self.assertFalse(an1 == an2)
        self.assertTrue(an1 != an2)
        self.assertTrue(an1 <= an2)
        self.assertTrue(an1 < an2)
        self.assertFalse(an1 >= an2)
        self.assertFalse(an1 > an2)
        # an1 ? en1
        self.assertFalse(an1 == en1)
        self.assertTrue(an1 != en1)
        self.assertTrue(an1 <= en1)
        self.assertTrue(an1 < en1)
        self.assertFalse(an1 >= en1)
        self.assertFalse(an1 > en1)
        # s1 ? an1
        self.assertTrue(s1 == an1)
        self.assertFalse(s1 != an1)
        self.assertTrue(s1 <= an1)
        self.assertFalse(s1 < an1)
        self.assertTrue(s1 >= an1)
        self.assertFalse(s1 > an1)
        # an1 ? a1
        self.assertTrue(an1 == s1)
        self.assertFalse(an1 != s1)
        self.assertTrue(an1 <= s1)
        self.assertFalse(an1 < s1)
        self.assertTrue(an1 >= s1)
        self.assertFalse(an1 > s1)

    class FakeDOM:
        namespaceURI = None
        localName = None

    def testConstructor (self):
        ln = 'local'
        ns_uri = 'urn:ns'
        en = ExpandedName(ln)
        self.assertEqual(en.namespace(), None)
        self.assertEqual(en.localName(), ln)
        en2 = ExpandedName(en)
        self.assertEqual(en2, en)
        dom = pyxb.utils.domutils.StringToDOM('<ns1:%s xmlns:ns1="%s" attr="52">content</ns1:%s>' % (ln, ns_uri, ln))
        en = ExpandedName(dom.documentElement)
        ns = pyxb.namespace.NamespaceForURI(ns_uri)
        self.assertTrue(ns is not None)
        self.assertEqual(ns, en.namespace())
        self.assertEqual(ln, en.localName())
        en2 = ExpandedName(ns, ln)
        self.assertEqual(en, en2)
        attr = dom.documentElement.getAttributeNodeNS(None, 'attr')
        self.assertTrue(attr is not None)
        en = ExpandedName(attr)
        self.assertEqual(xml.dom.EMPTY_NAMESPACE, en.namespaceURI())
        self.assertEqual('attr', en.localName())
        child = dom.documentElement.firstChild
        self.assertTrue(child is not None)
        self.assertEqual(xml.dom.Node.TEXT_NODE, child.nodeType)
        self.assertRaises(pyxb.LogicError, ExpandedName, child)

    def testMapping (self):
        an1 = ExpandedName(None, 'string')
        en1 = ExpandedName(xsd, 'string')
        en2 = ExpandedName(xsd, 'string')
        mymap = { }
        mymap[en1] = 'Yes'
        mymap[an1] = 'No'
        mymap['key'] = 'Key'
        self.assertEqual(mymap[en2], 'Yes')
        self.assertEqual(mymap[an1], 'No')
        self.assertEqual(mymap[an1.localName()], 'No')
        self.assertNotEqual(mymap[en2.localName()], 'Yes')
        self.assertEqual(mymap['key'], 'Key')
        self.assertEqual(mymap[ExpandedName(None, 'key')], 'Key')
        self.assertEqual(None, mymap.get('nokey'))
        del mymap[en2]
        self.assertEqual(None, mymap.get(en1))

    def testOrdering (self):
        s1 = "one"
        s2 = "two"
        en1 = ExpandedName(None, s1)
        en2 = ExpandedName(None, s2)
        xn1 = ExpandedName(xsd, s1)
        xn2 = ExpandedName(xsd, s2)
        self.assertTrue(s1 < s2)
        self.assertTrue(s2 > s1)
        self.assertTrue(en1 < s2)
        self.assertTrue(en2 > s1)

    def testAbsent (self):
        an = pyxb.namespace.CreateAbsentNamespace()
        an2 = pyxb.namespace.CreateAbsentNamespace()
        self.assertNotEqual(an, an2)
        self.assertEqual(an.uri(), an2.uri())
        ln = 'local'
        en1 = ExpandedName(None, ln)
        en2 = ExpandedName(an, ln)
        en3 = ExpandedName(an2, ln)
        self.assertEqual(en1, en2)
        self.assertEqual(en1, en3)
        self.assertEqual(en2, en3)
        self.assertEqual(hash(en1), hash(en2))
        self.assertEqual(hash(en1), hash(en3))
        self.assertEqual(hash(en2), hash(en3))

    def testCategoryDeferral (self):
        int_en = pyxb.namespace.ExpandedName(xsd, 'int')
        self.assertEqual(xsd_module.int, int_en.typeBinding())
        self.assertRaises(pyxb.NamespaceError, getattr, int_en, 'notACategory')

class TestCategories (unittest.TestCase):
    def testXSDCategories (self):
        # Need type and element bindings, along with all the component ones
        self.assertTrue('elementBinding' in xsd.categories())
        self.assertTrue('typeBinding' in xsd.categories())

    def testStandard (self):
        def_map = xsd.categoryMap('typeDefinition')
        binding_map = xsd.categoryMap('typeBinding')
        int_en = pyxb.namespace.ExpandedName(xsd, 'int')
        self.assertEqual(xsd_module.int, binding_map['int'])
        self.assertEqual(xsd_module.int.SimpleTypeDefinition(), def_map['int'])
        self.assertEqual(int_en.typeDefinition(), def_map['int'])
        self.assertEqual(xsd_module.int, int_en.typeBinding())

    def testNoCategory (self):
        self.assertRaises(pyxb.NamespaceError, pyxb.namespace.XMLSchema.categoryMap, 'not a category')

if '__main__' == __name__:
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.utils import six
import noi
import unittest
import pyxb
import io
import xml.dom.minidom

def first_element_child (n):
    for c in n.childNodes:
        if c.ELEMENT_NODE == c.nodeType:
            return c
    return None

class TestTrac0114 (unittest.TestCase):

    xmlData = open('namespace_other_issue.xml', 'rb').read()

    def validateDOM (self, instance):
        self.assertEqual
        print('a NS: %s / %s' % (a.namespaceURI, a.tagName))
        print('aext NS: %s / %s' % (aext.namespaceURI, aext.tagName))
        print('xyz NS: %s / %s' % (xyz.namespaceURI, xyz.tagName))

    def testMiniDOM (self):
        dom = xml.dom.minidom.parse(io.BytesIO(self.xmlData))
        a = dom.documentElement
        self.assertEqual(a.namespaceURI, noi.Namespace.uri())
        aext = first_element_child(a)
        self.assertEqual(aext.namespaceURI, noi.Namespace.uri())
        xyz = first_element_child(aext)
        self.assertEqual(xyz.namespaceURI, 'urn:schema:b')

    def testPyXB (self):
        instance = noi.CreateFromDocument(self.xmlData)
        dom = instance.toDOM()
        a = dom.documentElement
        self.assertEqual(a.namespaceURI, noi.Namespace.uri())
        aext = a.firstChild
        self.assertEqual(aext.namespaceURI, noi.Namespace.uri())
        xyz = aext.firstChild
        self.assertEqual(xyz.namespaceURI, 'urn:schema:b')
        xmlt = six.u('<ns1:a xmlns:ns1="urn:schema:a" xmlns:ns2="urn:schema:b" xmlns:ns3="urn:schema:c"><ns1:Extensions><ns2:xyz ns3:tag="value">abc</ns2:xyz></ns1:Extensions></ns1:a>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(a.toxml("utf-8"), xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
from pyxb.utils import six
import xml.dom.minidom
import io

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" targetNamespace="urn:trac-0074">
<xs:element name="top" type="xs:string"/>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0074 (unittest.TestCase):
    def test (self):
        t0p = Namespace.createExpandedName('t0p')
        xmlt = six.u('<ns:t0p xmlns:ns="urn:trac-0074">content</ns:t0p>')
        dom = xml.dom.minidom.parseString(xmlt)
        try:
            dom_instance = CreateFromDOM(dom.documentElement)
            self.fail('DOM creation succeeded')
        except pyxb.UnrecognizedDOMRootNodeError as e:
            self.assertEqual(dom.documentElement, e.node)
            self.assertEqual(t0p, e.node_name)

        saxdom = pyxb.utils.saxdom.parseString(xmlt)
        try:
            saxdom_instance = CreateFromDOM(saxdom)
            self.fail('SAXDOM creation succeeded')
        except pyxb.UnrecognizedDOMRootNodeError as e:
            self.assertEqual(saxdom.documentElement, e.node)
            self.assertEqual(t0p, e.node_name)

        saxer = pyxb.binding.saxer.make_parser()
        handler = saxer.getContentHandler()
        saxer.parse(io.StringIO(xmlt))
        try:
            sax_instance = handler.rootObject()
            self.fail('SAXER creation succeeded')
        except pyxb.UnrecognizedDOMRootNodeError as e:
            self.assertEqual(t0p, e.node_name)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
import app
import common4app

import pyxb.utils.domutils
from pyxb.utils import six

pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(app.Namespace, 'app')
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(common4app.Namespace, 'common')

class Test (unittest.TestCase):
    def testApp (self):
        x = common4app.extended('hi', 'there')
        a = app.elt(x)
        xmlt = six.u('<app:elt xmlns:app="urn:app" xmlns:common="urn:common"><app:xcommon><common:elt>hi</common:elt><common:extElt>there</common:extElt></app:xcommon></app:elt>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(a.toxml("utf-8", root_only=True), xmld)

if '__main__' == __name__:
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import TestPatternRestriction as t

xml="\U00010314"
p = t.test(xml)
print(p.toxml("utf-8"))



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="utf-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xs:simpleType name="tBase">
                <xs:restriction base="xs:normalizedString">
                        <xs:enumeration value="A"/>
                        <xs:enumeration value="B"/>
                        <xs:enumeration value="C"/>
                        <xs:enumeration value="D"/>
                </xs:restriction>
        </xs:simpleType>
        <xs:simpleType name="tRestr">
                <xs:restriction base="tBase"/>
        </xs:simpleType>
        <xs:simpleType name="tAltRestr">
                <xs:restriction base="tBase">
                    <xs:enumeration value="C"/>
    </xs:restriction>
        </xs:simpleType>
  <xs:element name="base" type="tBase"/>
  <xs:element name="restr" type="tRestr"/>
  <xs:element name="altrestr" type="tAltRestr"/>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0089 (unittest.TestCase):
    base_valid = ( 'A', 'B', 'C', 'D' )
    restr_valid = base_valid
    altrestr_valid = ( 'C' )
    invalid = ( 'Q' )

    def testBase (self):
        for ok in self.base_valid:
            xmls = '<base>%s</base>' % (ok,)
            instance = CreateFromDocument(xmls)
            self.assertEqual(instance, ok)
            self.assertEqual(instance, base(ok))
        for nok in self.invalid:
            xmls = '<base>%s</base>' % (nok,)
            self.assertRaises(pyxb.SimpleTypeValueError, CreateFromDocument, xmls)
            self.assertRaises(pyxb.SimpleTypeValueError, base, nok)

    def testRestr (self):
        for ok in self.restr_valid:
            xmls = '<base>%s</base>' % (ok,)
            instance = CreateFromDocument(xmls)
            self.assertEqual(instance, ok)
            self.assertEqual(instance, base(ok))
        for nok in self.invalid:
            xmls = '<base>%s</base>' % (nok,)
            self.assertRaises(pyxb.SimpleTypeValueError, CreateFromDocument, xmls)
            self.assertRaises(pyxb.SimpleTypeValueError, base, nok)

    def testAlt (self):
        xmls = '<altrestr>C</altrestr>'
        instance = CreateFromDocument(xmls)
        self.assertEqual(instance, 'C')
        self.assertEqual(instance, altrestr('C'))
        xmls = '<altrestr>A</altrestr>'
        self.assertRaises(pyxb.SimpleTypeValueError, CreateFromDocument, xmls)
        self.assertRaises(pyxb.SimpleTypeValueError, altrestr, 'A')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""This module contains support for Unicode characters as required to
support the regular expression syntax defined in U{annex F
<http://www/Documentation/W3C/www.w3.org/TR/xmlschema-2/index.html#regexs>}
of the XML Schema definition.

In particular, we need to be able to identify character properties and
block escapes, as defined in F.1.1, by name.

 - Block data: U{http://www.unicode.org/Public/3.1-Update/Blocks-4.txt}
 - Property list data: U{http://www.unicode.org/Public/3.1-Update/PropList-3.1.0.txt}
 - Full dataset: U{http://www.unicode.org/Public/3.1-Update/UnicodeData-3.1.0.txt}

The Unicode database active at the time XML Schema 1.0 was defined is
archived at
U{http://www.unicode.org/Public/3.1-Update/UnicodeCharacterDatabase-3.1.0.html},
and refers to U{Unicode Standard Annex #27: Unicode 3.1
<http://www.unicode.org/unicode/reports/tr27/>}.
"""

import re
import logging
import pyxb.utils.utility
from pyxb.utils import six
from pyxb.utils.six.moves import xrange

_log = logging.getLogger(__name__)

SupportsWideUnicode = False
try:
    re.compile(six.u('[\U0001d7ce-\U0001d7ff]'))
    SupportsWideUnicode = True
except:
    pass

import bisect

class CodePointSetError (LookupError):
    """Raised when some abuse of a L{CodePointSet} is detected."""
    pass

@pyxb.utils.utility.BackfillComparisons
class CodePointSet (object):
    """Represent a set of Unicode code points.

    Each code point is an integral value between 0 and 0x10FFFF.  This
    class is used to represent a set of code points in a manner
    suitable for use as regular expression character sets."""

    MaxCodePoint = 0x10FFFF
    """The maximum value for a code point in the Unicode code point
    space.  This is normally 0xFFFF, because wide unicode characters
    are generally not enabled in Python builds.  If, however, they are
    enabled, this will be the full value of 0x10FFFF."""

    MaxShortCodePoint = 0xFFFF
    if not SupportsWideUnicode:
        MaxCodePoint = MaxShortCodePoint

    # The internal representation of the codepoints is as a sorted
    # list where values at an even index denote the first codepoint in
    # a range that is in the set, and the immediately following value
    # indicates the next following codepoint that is not in the set.
    # A missing value at the end is interpreted as MaxCodePoint.  For
    # example, the sequence [ 12, 15, 200 ] denotes the set containing
    # codepoints 12, 13, 14, and everything above 199.
    __codepoints = None

    def _codepoints (self):
        """For testing purrposes only, access to the codepoints
        internal representation."""
        return self.__codepoints

    def __hash__ (self):
        return hash(self.__codepoints)

    def __eq__ (self, other):
        """Equality is delegated to the codepoints list."""
        return self.__codepoints == other.__codepoints

    def __lt__ (self, other):
        return self.__codepoints < other.__codepoints

    def __init__ (self, *args):
        self.__codepoints = []
        if 1 == len(args):
            if isinstance(args[0], CodePointSet):
                self.__codepoints.extend(args[0].__codepoints)
                return
            if isinstance(args[0], list):
                args = args[0]
        for a in args:
            self.add(a)

    def __mutate (self, value, do_add):
        # Identify the start (inclusive) and end (exclusive) code
        # points of the value's range.
        if isinstance(value, tuple):
            (s, e) = value
            e += 1
        elif isinstance(value, six.string_types):
            if 1 < len(value):
                raise TypeError()
            s = ord(value)
            e = s+1
        else:
            s = int(value)
            e = s+1
        if s >= e:
            raise ValueError('codepoint range value order')

        # Validate the range for the code points supported by this
        # Python interpreter.  Recall that e is exclusive.
        if s > self.MaxCodePoint:
            return self
        if e > self.MaxCodePoint:
            e = self.MaxCodePoint+1

        # Index of first code point equal to or greater than s
        li = bisect.bisect_left(self.__codepoints, s)
        # Index of last code point less than or equal to e
        ri = bisect.bisect_right(self.__codepoints, e)
        # There are four cases; if we're subtracting, they reflect.
        case = ((li & 1) << 1) | (ri & 1)
        if not do_add:
            case = 3 - case
        if 0x03 == case:
            # Add: Incoming value begins and ends within existing ranges
            del self.__codepoints[li:ri]
        elif 0x02 == case:
            # Add: Incoming value extends into an excluded range
            del self.__codepoints[li+1:ri]
            self.__codepoints[li] = e
        elif 0x01 == case:
            # Add: Incoming value begins in an excluded range
            del self.__codepoints[li+1:ri]
            self.__codepoints[li] = s
        else:
            # Add: Incoming value begins and ends within excluded ranges
            self.__codepoints[li:ri] = [s, e]
        return self

    def add (self, value):
        """Add the given value to the code point set.

        @param value: An integral value denoting a code point, or a
        tuple C{(s,e)} denoting the start and end (inclusive) code
        points in a range.
        @return: C{self}"""
        return self.__mutate(value, True)

    def extend (self, values):
        """Add multiple values to a code point set.

        @param values: Either a L{CodePointSet} instance, or an iterable
        whose members are valid parameters to L{add}.

        @return: C{self}"""
        if isinstance(values, CodePointSet):
            self.extend(values.asTuples())
        else:
            for v in values:
                self.__mutate(v, True)
        return self

    def subtract (self, value):
        """Remove the given value from the code point set.

        @param value: An integral value denoting a code point, or a tuple
        C{(s,e)} denoting the start and end (inclusive) code points in a
        range, or a L{CodePointSet}.

        @return: C{self}"""
        if isinstance(value, CodePointSet):
            for v in value.asTuples():
                self.subtract(v)
            return self
        return self.__mutate(value, False)

    # Escape sequences for characters that must not appear unescaped in
    # Python regular expression patterns.  Maps each bad character to a safe
    # escape sequence.
    __XMLtoPythonREEscapedCodepoints = (
        # From docs for Python's "re" module: Regular expression
        # pattern strings may not contain null bytes
        0,
        # Indicates negation if it happens to occur at the start of a
        # character group
        ord('^'),
        # Escape character (backslash)
        ord('\\'),
        # Actually doesn't need to be escaped inside a Python
        # character group, but escaping it is less confusing.
        ord('['),
        # End of character group
        ord(']'),
        # Indicates a range of characters
        ord('-')
    )

    # Return the given code point as a unicode character suitable for
    # use in a regular expression
    def __unichr (self, code_point):
        rv = six.unichr(code_point)
        if 0 == code_point:
            rv = six.u('x00')
        if code_point in self.__XMLtoPythonREEscapedCodepoints:
            rv = six.unichr(0x5c) + rv
        return rv

    def asPattern (self, with_brackets=True):
        """Return the code point set as Unicode regular expression
        character group consisting of a sequence of characters or
        character ranges.

        This returns a regular expression fragment using Python's
        regular expression syntax.  Note that different regular expression
        syntaxes are not compatible, often in subtle ways.

        @param with_brackets: If C{True} (default), square brackets
        are added to enclose the returned character group."""
        rva = []
        if with_brackets:
            rva.append(six.u('['))
        for (s, e) in self.asTuples():
            if s == e:
                rva.append(self.__unichr(s))
            else:
                rva.extend([self.__unichr(s), '-', self.__unichr(e)])
        if with_brackets:
            rva.append(six.u(']'))
        return six.u('').join(rva)

    def asTuples (self):
        """Return the codepoints as tuples denoting the ranges that are in
        the set.

        Each tuple C{(s, e)} indicates that the code points from C{s}
        (inclusive) to C{e}) (inclusive) are in the set."""

        rv = []
        start = None
        for ri in xrange(len(self.__codepoints)):
            if start is not None:
                rv.append( (start, self.__codepoints[ri]-1) )
                start = None
            else:
                start = self.__codepoints[ri]
        if (start is not None) and (start <= self.MaxCodePoint):
            rv.append( (start, self.MaxCodePoint) )
        return rv

    def negate (self):
        """Return an instance that represents the inverse of this set."""
        rv = type(self)()
        if (0 < len(self.__codepoints)) and (0 == self.__codepoints[0]):
            rv.__codepoints.extend(self.__codepoints[1:])
        else:
            rv.__codepoints.append(0)
            rv.__codepoints.extend(self.__codepoints)
        return rv

    def asSingleCharacter (self):
        """If this set represents a single character, return it as its
        unicode string value.  Otherwise return C{None}."""
        if (2 != len(self.__codepoints)) or (1 < (self.__codepoints[1] - self.__codepoints[0])):
            return None
        return six.unichr(self.__codepoints[0])

from pyxb.utils.unicode_data import PropertyMap
from pyxb.utils.unicode_data import BlockMap

class XML1p0e2 (object):
    """Regular expression support for XML Schema Data Types.

    This class holds character classes and regular expressions used to
    constrain the lexical space of XML Schema datatypes derived from
    U{string<http://www.w3.org/TR/xmlschema-2/#string>}.  They are
    from U{XML 1.0 (Second
    Edition)<http://www.w3.org/TR/2000/WD-xml-2e-20000814>} and
    U{Namespaces in XML
    <http://www.w3.org/TR/1999/REC-xml-names-19990114/>}.

    Unlike the regular expressions used for pattern constraints in XML
    Schema, which are derived from the Unicode 3.1 specification,
    these are derived from the Unicode 2.0 specification.

    The XML Schema definition refers explicitly to the second edition
    of XML, so we have to use these code point sets and patterns.  Be
    aware that U{subsequent updates to the XML specification
    <http://www.w3.org/XML/xml-V10-4e-errata#E09>} have changed the
    corresponding patterns for other uses of XML.  One significant
    change is that the original specification, used here, does not
    allow wide unicode characters."""

    Char = CodePointSet(
        0x0009,
        0x000A,
        0x000D,
        ( 0x0020, 0xD7FF ),
        ( 0xE000, 0xFFFD )
        )
    if SupportsWideUnicode:
        Char.add( ( 1+CodePointSet.MaxShortCodePoint, CodePointSet.MaxCodePoint ) )

    BaseChar = CodePointSet(
        ( 0x0041, 0x005A ),
        ( 0x0061, 0x007A ),
        ( 0x00C0, 0x00D6 ),
        ( 0x00D8, 0x00F6 ),
        ( 0x00F8, 0x00FF ),
        ( 0x0100, 0x0131 ),
        ( 0x0134, 0x013E ),
        ( 0x0141, 0x0148 ),
        ( 0x014A, 0x017E ),
        ( 0x0180, 0x01C3 ),
        ( 0x01CD, 0x01F0 ),
        ( 0x01F4, 0x01F5 ),
        ( 0x01FA, 0x0217 ),
        ( 0x0250, 0x02A8 ),
        ( 0x02BB, 0x02C1 ),
        0x0386,
        ( 0x0388, 0x038A ),
        0x038C,
        ( 0x038E, 0x03A1 ),
        ( 0x03A3, 0x03CE ),
        ( 0x03D0, 0x03D6 ),
        0x03DA,
        0x03DC,
        0x03DE,
        0x03E0,
        ( 0x03E2, 0x03F3 ),
        ( 0x0401, 0x040C ),
        ( 0x040E, 0x044F ),
        ( 0x0451, 0x045C ),
        ( 0x045E, 0x0481 ),
        ( 0x0490, 0x04C4 ),
        ( 0x04C7, 0x04C8 ),
        ( 0x04CB, 0x04CC ),
        ( 0x04D0, 0x04EB ),
        ( 0x04EE, 0x04F5 ),
        ( 0x04F8, 0x04F9 ),
        ( 0x0531, 0x0556 ),
        0x0559,
        ( 0x0561, 0x0586 ),
        ( 0x05D0, 0x05EA ),
        ( 0x05F0, 0x05F2 ),
        ( 0x0621, 0x063A ),
        ( 0x0641, 0x064A ),
        ( 0x0671, 0x06B7 ),
        ( 0x06BA, 0x06BE ),
        ( 0x06C0, 0x06CE ),
        ( 0x06D0, 0x06D3 ),
        0x06D5,
        ( 0x06E5, 0x06E6 ),
        ( 0x0905, 0x0939 ),
        0x093D,
        ( 0x0958, 0x0961 ),
        ( 0x0985, 0x098C ),
        ( 0x098F, 0x0990 ),
        ( 0x0993, 0x09A8 ),
        ( 0x09AA, 0x09B0 ),
        0x09B2,
        ( 0x09B6, 0x09B9 ),
        ( 0x09DC, 0x09DD ),
        ( 0x09DF, 0x09E1 ),
        ( 0x09F0, 0x09F1 ),
        ( 0x0A05, 0x0A0A ),
        ( 0x0A0F, 0x0A10 ),
        ( 0x0A13, 0x0A28 ),
        ( 0x0A2A, 0x0A30 ),
        ( 0x0A32, 0x0A33 ),
        ( 0x0A35, 0x0A36 ),
        ( 0x0A38, 0x0A39 ),
        ( 0x0A59, 0x0A5C ),
        0x0A5E,
        ( 0x0A72, 0x0A74 ),
        ( 0x0A85, 0x0A8B ),
        0x0A8D,
        ( 0x0A8F, 0x0A91 ),
        ( 0x0A93, 0x0AA8 ),
        ( 0x0AAA, 0x0AB0 ),
        ( 0x0AB2, 0x0AB3 ),
        ( 0x0AB5, 0x0AB9 ),
        0x0ABD,
        0x0AE0,
        ( 0x0B05, 0x0B0C ),
        ( 0x0B0F, 0x0B10 ),
        ( 0x0B13, 0x0B28 ),
        ( 0x0B2A, 0x0B30 ),
        ( 0x0B32, 0x0B33 ),
        ( 0x0B36, 0x0B39 ),
        0x0B3D,
        ( 0x0B5C, 0x0B5D ),
        ( 0x0B5F, 0x0B61 ),
        ( 0x0B85, 0x0B8A ),
        ( 0x0B8E, 0x0B90 ),
        ( 0x0B92, 0x0B95 ),
        ( 0x0B99, 0x0B9A ),
        0x0B9C,
        ( 0x0B9E, 0x0B9F ),
        ( 0x0BA3, 0x0BA4 ),
        ( 0x0BA8, 0x0BAA ),
        ( 0x0BAE, 0x0BB5 ),
        ( 0x0BB7, 0x0BB9 ),
        ( 0x0C05, 0x0C0C ),
        ( 0x0C0E, 0x0C10 ),
        ( 0x0C12, 0x0C28 ),
        ( 0x0C2A, 0x0C33 ),
        ( 0x0C35, 0x0C39 ),
        ( 0x0C60, 0x0C61 ),
        ( 0x0C85, 0x0C8C ),
        ( 0x0C8E, 0x0C90 ),
        ( 0x0C92, 0x0CA8 ),
        ( 0x0CAA, 0x0CB3 ),
        ( 0x0CB5, 0x0CB9 ),
        0x0CDE,
        ( 0x0CE0, 0x0CE1 ),
        ( 0x0D05, 0x0D0C ),
        ( 0x0D0E, 0x0D10 ),
        ( 0x0D12, 0x0D28 ),
        ( 0x0D2A, 0x0D39 ),
        ( 0x0D60, 0x0D61 ),
        ( 0x0E01, 0x0E2E ),
        0x0E30,
        ( 0x0E32, 0x0E33 ),
        ( 0x0E40, 0x0E45 ),
        ( 0x0E81, 0x0E82 ),
        0x0E84,
        ( 0x0E87, 0x0E88 ),
        0x0E8A,
        0x0E8D,
        ( 0x0E94, 0x0E97 ),
        ( 0x0E99, 0x0E9F ),
        ( 0x0EA1, 0x0EA3 ),
        0x0EA5,
        0x0EA7,
        ( 0x0EAA, 0x0EAB ),
        ( 0x0EAD, 0x0EAE ),
        0x0EB0,
        ( 0x0EB2, 0x0EB3 ),
        0x0EBD,
        ( 0x0EC0, 0x0EC4 ),
        ( 0x0F40, 0x0F47 ),
        ( 0x0F49, 0x0F69 ),
        ( 0x10A0, 0x10C5 ),
        ( 0x10D0, 0x10F6 ),
        0x1100,
        ( 0x1102, 0x1103 ),
        ( 0x1105, 0x1107 ),
        0x1109,
        ( 0x110B, 0x110C ),
        ( 0x110E, 0x1112 ),
        0x113C,
        0x113E,
        0x1140,
        0x114C,
        0x114E,
        0x1150,
        ( 0x1154, 0x1155 ),
        0x1159,
        ( 0x115F, 0x1161 ),
        0x1163,
        0x1165,
        0x1167,
        0x1169,
        ( 0x116D, 0x116E ),
        ( 0x1172, 0x1173 ),
        0x1175,
        0x119E,
        0x11A8,
        0x11AB,
        ( 0x11AE, 0x11AF ),
        ( 0x11B7, 0x11B8 ),
        0x11BA,
        ( 0x11BC, 0x11C2 ),
        0x11EB,
        0x11F0,
        0x11F9,
        ( 0x1E00, 0x1E9B ),
        ( 0x1EA0, 0x1EF9 ),
        ( 0x1F00, 0x1F15 ),
        ( 0x1F18, 0x1F1D ),
        ( 0x1F20, 0x1F45 ),
        ( 0x1F48, 0x1F4D ),
        ( 0x1F50, 0x1F57 ),
        0x1F59,
        0x1F5B,
        0x1F5D,
        ( 0x1F5F, 0x1F7D ),
        ( 0x1F80, 0x1FB4 ),
        ( 0x1FB6, 0x1FBC ),
        0x1FBE,
        ( 0x1FC2, 0x1FC4 ),
        ( 0x1FC6, 0x1FCC ),
        ( 0x1FD0, 0x1FD3 ),
        ( 0x1FD6, 0x1FDB ),
        ( 0x1FE0, 0x1FEC ),
        ( 0x1FF2, 0x1FF4 ),
        ( 0x1FF6, 0x1FFC ),
        0x2126,
        ( 0x212A, 0x212B ),
        0x212E,
        ( 0x2180, 0x2182 ),
        ( 0x3041, 0x3094 ),
        ( 0x30A1, 0x30FA ),
        ( 0x3105, 0x312C ),
        ( 0xAC00, 0xD7A3 )
        )

    Ideographic = CodePointSet(
        ( 0x4E00, 0x9FA5 ),
        0x3007,
        ( 0x3021, 0x3029 )
        )

    Letter = CodePointSet(BaseChar).extend(Ideographic)

    CombiningChar = CodePointSet(
        ( 0x0300, 0x0345 ),
        ( 0x0360, 0x0361 ),
        ( 0x0483, 0x0486 ),
        ( 0x0591, 0x05A1 ),
        ( 0x05A3, 0x05B9 ),
        ( 0x05BB, 0x05BD ),
        0x05BF,
        ( 0x05C1, 0x05C2 ),
        0x05C4,
        ( 0x064B, 0x0652 ),
        0x0670,
        ( 0x06D6, 0x06DC ),
        ( 0x06DD, 0x06DF ),
        ( 0x06E0, 0x06E4 ),
        ( 0x06E7, 0x06E8 ),
        ( 0x06EA, 0x06ED ),
        ( 0x0901, 0x0903 ),
        0x093C,
        ( 0x093E, 0x094C ),
        0x094D,
        ( 0x0951, 0x0954 ),
        ( 0x0962, 0x0963 ),
        ( 0x0981, 0x0983 ),
        0x09BC,
        0x09BE,
        0x09BF,
        ( 0x09C0, 0x09C4 ),
        ( 0x09C7, 0x09C8 ),
        ( 0x09CB, 0x09CD ),
        0x09D7,
        ( 0x09E2, 0x09E3 ),
        0x0A02,
        0x0A3C,
        0x0A3E,
        0x0A3F,
        ( 0x0A40, 0x0A42 ),
        ( 0x0A47, 0x0A48 ),
        ( 0x0A4B, 0x0A4D ),
        ( 0x0A70, 0x0A71 ),
        ( 0x0A81, 0x0A83 ),
        0x0ABC,
        ( 0x0ABE, 0x0AC5 ),
        ( 0x0AC7, 0x0AC9 ),
        ( 0x0ACB, 0x0ACD ),
        ( 0x0B01, 0x0B03 ),
        0x0B3C,
        ( 0x0B3E, 0x0B43 ),
        ( 0x0B47, 0x0B48 ),
        ( 0x0B4B, 0x0B4D ),
        ( 0x0B56, 0x0B57 ),
        ( 0x0B82, 0x0B83 ),
        ( 0x0BBE, 0x0BC2 ),
        ( 0x0BC6, 0x0BC8 ),
        ( 0x0BCA, 0x0BCD ),
        0x0BD7,
        ( 0x0C01, 0x0C03 ),
        ( 0x0C3E, 0x0C44 ),
        ( 0x0C46, 0x0C48 ),
        ( 0x0C4A, 0x0C4D ),
        ( 0x0C55, 0x0C56 ),
        ( 0x0C82, 0x0C83 ),
        ( 0x0CBE, 0x0CC4 ),
        ( 0x0CC6, 0x0CC8 ),
        ( 0x0CCA, 0x0CCD ),
        ( 0x0CD5, 0x0CD6 ),
        ( 0x0D02, 0x0D03 ),
        ( 0x0D3E, 0x0D43 ),
        ( 0x0D46, 0x0D48 ),
        ( 0x0D4A, 0x0D4D ),
        0x0D57,
        0x0E31,
        ( 0x0E34, 0x0E3A ),
        ( 0x0E47, 0x0E4E ),
        0x0EB1,
        ( 0x0EB4, 0x0EB9 ),
        ( 0x0EBB, 0x0EBC ),
        ( 0x0EC8, 0x0ECD ),
        ( 0x0F18, 0x0F19 ),
        0x0F35,
        0x0F37,
        0x0F39,
        0x0F3E,
        0x0F3F,
        ( 0x0F71, 0x0F84 ),
        ( 0x0F86, 0x0F8B ),
        ( 0x0F90, 0x0F95 ),
        0x0F97,
        ( 0x0F99, 0x0FAD ),
        ( 0x0FB1, 0x0FB7 ),
        0x0FB9,
        ( 0x20D0, 0x20DC ),
        0x20E1,
        ( 0x302A, 0x302F ),
        0x3099,
        0x309A
        )

    Digit = CodePointSet(
        ( 0x0030, 0x0039 ),
        ( 0x0660, 0x0669 ),
        ( 0x06F0, 0x06F9 ),
        ( 0x0966, 0x096F ),
        ( 0x09E6, 0x09EF ),
        ( 0x0A66, 0x0A6F ),
        ( 0x0AE6, 0x0AEF ),
        ( 0x0B66, 0x0B6F ),
        ( 0x0BE7, 0x0BEF ),
        ( 0x0C66, 0x0C6F ),
        ( 0x0CE6, 0x0CEF ),
        ( 0x0D66, 0x0D6F ),
        ( 0x0E50, 0x0E59 ),
        ( 0x0ED0, 0x0ED9 ),
        ( 0x0F20, 0x0F29 )
        )

    Extender = CodePointSet(
        0x00B7,
        0x02D0,
        0x02D1,
        0x0387,
        0x0640,
        0x0E46,
        0x0EC6,
        0x3005,
        ( 0x3031, 0x3035 ),
        ( 0x309D, 0x309E ),
        ( 0x30FC, 0x30FE )
        )

    # Not an explicit production, but used in Name production
    NameStartChar = CodePointSet(Letter)
    NameStartChar.add(ord('_'))
    NameStartChar.add(ord(':'))

    NCNameStartChar = CodePointSet(Letter)
    NCNameStartChar.add(ord('_'))

    NameChar = CodePointSet(Letter)
    NameChar.extend(Digit)
    NameChar.add(ord('.'))
    NameChar.add(ord('-'))
    NameChar.add(ord('_'))
    NameChar.add(ord(':'))
    NameChar.extend(CombiningChar)
    NameChar.extend(Extender)

    NCNameChar = CodePointSet(Letter)
    NCNameChar.extend(Digit)
    NCNameChar.add(ord('.'))
    NCNameChar.add(ord('-'))
    NCNameChar.add(ord('_'))
    NCNameChar.extend(CombiningChar)
    NCNameChar.extend(Extender)

    Name_pat = '%s%s*' % (NameStartChar.asPattern(), NameChar.asPattern())
    Name_re = re.compile('^%s$' % (Name_pat,))
    NmToken_pat = '%s+' % (NameChar.asPattern(),)
    NmToken_re = re.compile('^%s$' % (NmToken_pat,))
    NCName_pat = '%s%s*' % (NCNameStartChar.asPattern(), NCNameChar.asPattern())
    NCName_re = re.compile('^%s$' % (NCName_pat,))
    QName_pat = '(%s:)?%s' % (NCName_pat, NCName_pat)
    QName_re = re.compile('^%s$' % (QName_pat,))

# Production 24 : Single Character Escapes
SingleCharEsc = { 'n' : CodePointSet(0x0A),
                  'r' : CodePointSet(0x0D),
                  't' : CodePointSet(0x09) }
for c in r'\|.-^?*+{}()[]':
    SingleCharEsc[c] = CodePointSet(ord(c))

# Production 25 : Category Escapes
# Production 26: Complemented Category Escapes
catEsc = { }
complEsc = { }
for k, v in six.iteritems(PropertyMap):
    catEsc[six.u('p{%s}') % (k,)] = v
    catEsc[six.u('P{%s}') % (k,)] = v.negate()

# Production 36 : IsBlock escapes
IsBlockEsc = { }
for k, v in six.iteritems(BlockMap):
    IsBlockEsc[six.u('p{Is%s}') % (k,)] = v
    IsBlockEsc[six.u('P{Is%s}') % (k,)] = v.negate()

# Production 37 : Multi-Character Escapes
WildcardEsc = CodePointSet(ord('\n'), ord('\r')).negate()
MultiCharEsc = { }
MultiCharEsc['s'] = CodePointSet(0x20, ord('\t'), ord('\n'), ord('\r'))
MultiCharEsc['S'] = MultiCharEsc['s'].negate()
MultiCharEsc['i'] = CodePointSet(XML1p0e2.Letter).add(ord('_')).add(ord(':'))
MultiCharEsc['I'] = MultiCharEsc['i'].negate()
MultiCharEsc['c'] = CodePointSet(XML1p0e2.NameChar)
MultiCharEsc['C'] = MultiCharEsc['c'].negate()
MultiCharEsc['d'] = PropertyMap['Nd']
MultiCharEsc['D'] = MultiCharEsc['d'].negate()
MultiCharEsc['W'] = CodePointSet(PropertyMap['P']).extend(PropertyMap['Z']).extend(PropertyMap['C'])
MultiCharEsc['w'] = MultiCharEsc['W'].negate()



import pyxb
import normal as custom
import raw.custom as raw_custom

import unittest

class TestComplex (unittest.TestCase):

    def setUp (self):
        xmls = open('test.xml').read()
        self.instance = custom.CreateFromDocument(xmls)

    def testRawSubclassHierarchy (self):
        self.assertTrue(issubclass(raw_custom.tc01, raw_custom.ta0))
        self.assertTrue(issubclass(raw_custom.tc02, raw_custom.ta0))
        self.assertTrue(issubclass(raw_custom.tc03, raw_custom.ta0))
        self.assertTrue(issubclass(raw_custom.ta04, raw_custom.ta0))
        self.assertTrue(issubclass(raw_custom.tc041, raw_custom.ta04))
        self.assertTrue(issubclass(raw_custom.tc042, raw_custom.ta04))

    def testCustomSubclassesRaw(self):
        self.assertTrue(issubclass(custom.tc01, raw_custom.tc01))
        self.assertTrue(issubclass(custom.tc02, raw_custom.tc02))
        self.assertTrue(issubclass(custom.tc03, raw_custom.tc03))
        self.assertTrue(issubclass(custom.ta04, raw_custom.ta04))
        self.assertTrue(issubclass(custom.tc041, raw_custom.tc041))
        self.assertTrue(issubclass(custom.tc042, raw_custom.tc042))

    def testCustomConcreteHierarchy(self):
        self.assertFalse(issubclass(custom.tc01, custom.ta0))
        self.assertTrue(issubclass(custom.tc02, custom.ta0))
        self.assertFalse(issubclass(custom.tc03, custom.ta0))

    def test_c01 (self):
        ec01 = self.instance.ec01
        self.assertTrue(isinstance(ec01, custom.tc01))
        self.assertEqual(ec01.ea0, 'ec01')
        self.assertEqual(ec01.ec01, 'c01')
        # Direct customization works...
        self.assertEqual(ec01.xc01(), 'extend tc01')
        # No inheritance from customized superclass
        self.assertRaises(AttributeError, lambda _i: _i.xa0, ec01)

    def test_c02 (self):
        # Dual-inheritance customization works
        ec02 = self.instance.ec02
        self.assertTrue(isinstance(ec02, custom.tc02))
        self.assertEqual(ec02.ea0, 'ec02')
        self.assertEqual(ec02.ec02_i, 2)
        # Direct customization works
        self.assertEqual(ec02.xc02(), 'extend tc02')
        # Inherited customization works
        self.assertEqual(ec02.xa0(), 'extend ta0')

    def test_c03 (self):
        ec03 = self.instance.ec03
        self.assertTrue(isinstance(ec03, custom.tc03))
        self.assertEqual(ec03.ea0, 'ec03')
        self.assertTrue(ec03.ec03_b)
        # No inheritance from customized superclass
        self.assertRaises(AttributeError, lambda _i: _i.xa0, ec03)

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
from __future__ import print_function
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import sys
sys.path.append("modules")

import com.example.pyxb.ModelA.AA
import com.example.pyxb.ModelB.BB

print('Modules loaded')
sys.exit(0)



# -*- coding: utf-8 -*-
from __future__ import print_function
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import unittest
import pyxb.binding.datatypes as xsd
from pyxb.utils.utility import UTCOffsetTimeZone
import datetime

class Test_dateTime (unittest.TestCase):

    Canonical = datetime.datetime(2002, 10, 27, 12, 14, 32, 123400, None)

    def verifyTime (self, dt, with_usec=True, with_adj=(0,0), with_tzinfo=True):
        self.assertEqual(self.Canonical.year, dt.year)
        self.assertEqual(self.Canonical.month, dt.month)
        self.assertEqual(self.Canonical.day, dt.day)
        (hour_adj, minute_adj) = with_adj
        self.assertEqual(self.Canonical.hour + hour_adj, dt.hour)
        self.assertEqual(self.Canonical.minute + minute_adj, dt.minute)
        self.assertEqual(self.Canonical.second, dt.second)
        if with_usec:
            self.assertEqual(self.Canonical.microsecond, dt.microsecond)
        self.assertEqual(with_tzinfo, dt.tzinfo is not None)

    def testBad (self):
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.dateTime, '2002-10-27 12:14:32  ')
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.dateTime, '2002-10-27 12:14:32.Z')
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.dateTime, '2002-10-27 12:14:32.123405:00')
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.dateTime, '2002-10-27 12:14:32.1234+05')

    def testFromText (self):
        self.verifyTime(xsd.dateTime('  2002-10-27T12:14:32', _from_xml=True), with_usec=False, with_tzinfo=False)
        self.verifyTime(xsd.dateTime('2002-10-27T12:14:32  ', _from_xml=True), with_usec=False, with_tzinfo=False)
        self.verifyTime(xsd.dateTime('2002-10-27T12:14:32'), with_usec=False, with_tzinfo=False)
        self.verifyTime(xsd.dateTime('2002-10-27T12:14:32.1234'), with_tzinfo=False)
        self.verifyTime(xsd.dateTime('2002-10-27T12:14:32Z'), with_usec=False)
        self.verifyTime(xsd.dateTime('2002-10-27T12:14:32.1234Z'))
        self.verifyTime(xsd.dateTime('2002-10-27T12:14:32.1234+05:00'), with_adj=(-5,0))
        self.verifyTime(xsd.dateTime('2002-10-27T12:14:32.1234Z'))

    def testYear (self):
        # This test can't succeed because Python doesn't support negative years.
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.dateTime, '-0024-01-01T00:00:00')

    def testXsdLiteral (self):
        dt = xsd.dateTime('2002-10-27T12:14:32Z')
        self.assertEqual('2002-10-27T12:14:32Z', dt.xsdLiteral())
        self.assertTrue(dt.tzinfo is not None)
        self.assertEqual('2002-10-27T07:14:32Z', xsd.dateTime('2002-10-27T12:14:32+05:00').xsdLiteral())
        self.assertEqual('2002-10-27T17:14:32Z', xsd.dateTime('2002-10-27T12:14:32-05:00').xsdLiteral())
        self.assertEqual('2002-10-27T17:14:32.1234Z', xsd.dateTime('2002-10-27T12:14:32.123400-05:00').xsdLiteral())
        # No zone info
        dt = xsd.dateTime('2002-10-27T12:14:32')
        self.assertEqual('2002-10-27T12:14:32', dt.xsdLiteral())
        self.assertFalse(dt.tzinfo is not None)

    def testNormalize (self):
        dt = xsd.dateTime('2000-03-04T23:00:00+03:00')
        self.assertEqual('2000-03-04T20:00:00Z', dt.xsdLiteral())

        pdt = datetime.datetime(year=2000, month=3, day=4, hour=23, tzinfo=UTCOffsetTimeZone(180))
        self.assertEqual('2000-03-04 23:00:00+03:00', str(pdt))
        dt = xsd.dateTime(pdt)
        self.assertEqual('2000-03-04T20:00:00Z', dt.xsdLiteral())

        dt = xsd.dateTime(2000, 3, 4, 23, tzinfo=UTCOffsetTimeZone(180))
        self.assertEqual('2000-03-04T20:00:00Z', dt.xsdLiteral())

    # Manual test to see whether LocalTime works; run this on a
    # machine that uses DST.
    def XtestBogus (self):
        dt = xsd.dateTime.today()
        print(dt.xsdLiteral())
        print(str(dt))
        print(dt.aslocal())
        # NB: duration does not support months in Python version
        delta = xsd.duration('P%dD' % (365 / 2))
        dt = xsd.dateTime(dt + delta)
        print(dt.xsdLiteral())
        print(str(dt))
        print(dt.aslocal())


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.binding.generate
import pyxb.utils.domutils

from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-typeinf.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

import unittest

class TestTypeInference (unittest.TestCase):
    def testBasic (self):
        e = anyType(4) # should be int
        self.assertEqual(e.int, 4)
        self.assertTrue(e.float is None)
        self.assertTrue(e.str is None)
        e = anyType(4.4)
        self.assertTrue(e.int is None)
        self.assertEqual(e.float, 4.4)
        self.assertTrue(e.str is None)
        e = anyType("3")
        self.assertTrue(e.int is None)
        self.assertTrue(e.float is None)
        self.assertEqual(e.str, "3")

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""This module contains support for processing XML using a SAX parser.

In particular, it provides a L{base content handler class<BaseSAXHandler>}
that maintains namespace context and element state in a stack; and a L{base
element state class <SAXElementState>} which records the location of the
element in the stream.  These classes are extended for specific parsing needs
(e.g., L{pyxb.binding.saxer}).
"""

from __future__ import print_function
import xml.sax
import xml.sax.handler
import io
import logging
import pyxb.namespace
from pyxb.utils import six

_log = logging.getLogger(__name__)

class TracingSAXHandler (xml.sax.handler.ContentHandler):
    """A SAX handler class which prints each method invocation.
    """

    # Whether invocation of handler methods should be traced
    __trace = False

    def setDocumentLocator (self, locator):
        print('setDocumentLocator %s' % (locator,))

    def startDocument (self):
        print('startDocument')

    def startPrefixMapping (self, prefix, uri):
        print('startPrefixMapping %s %s' % (prefix, uri))

    def endPrefixMapping (self, prefix):
        print('endPrefixMapping %s' % (prefix,))

    def startElementNS (self, name, qname, attrs):
        print('startElementNS %s %s' % (name, qname))

    def endElementNS (self, name, qname):
        print('endElementNS %s %s' % (name, qname))

    def characters (self, content):
        print('characters %s' % (content,))

    def ignorableWhitespace (self, whitespace):
        print('ignorableWhitespace len %d' % (len(whitespace),))

    def processingInstruction (self, target, data):
        print('processingInstruction %s %s' % (target, data))

class _NoopSAXHandler (xml.sax.handler.ContentHandler):
    """A SAX handler class which doesn't do anything.  Used to get baseline
    performance parsing a particular document.
    """

    def setDocumentLocator (self, locator):
        pass

    def startDocument (self):
        pass

    def startPrefixMapping (self, prefix, uri):
        pass

    def endPrefixMapping (self, prefix):
        pass

    def startElementNS (self, name, qname, attrs):
        pass

    def endElementNS (self, name, qname):
        pass

    def characters (self, content):
        pass

    def ignorableWhitespace (self, whitespace):
        pass

    def processingInstruction (self, target, data):
        pass

class SAXInformationItem (object):
    """Class used to capture an item discovered in the body of an element."""

    location = None
    """Where the item began in the document."""

    item = None
    """The item.  Generally either character information (as text) or a DOM
    Node instance or a binding instance."""

    maybe_element = None
    """C{False} iff the L{item} is character information as opposed to element content."""

    element_decl = None
    """A reference to the
    L{ElementDeclaration<pyxb.binding.content.ElementDeclaration>} used for
    the L{item}.  This will be C{None} for element content that does not have
    an enclosing CTD scope."""

    def __init__ (self, location, item, maybe_element, element_decl=None):
        self.location = location
        self.item = item
        self.maybe_element = maybe_element
        self.element_decl = element_decl

class SAXElementState (object):
    """State corresponding to processing a given element with the SAX
    model."""

    def contentHandler (self):
        """Reference to the C{xml.sxa.handler.ContentHandler} that is processing the document."""
        return self.__contentHandler
    __contentHandler = None

    def parentState (self):
        """Reference to the SAXElementState of the element enclosing this
        one."""
        return self.__parentState
    __parentState = None

    def namespaceContext (self):
        """The L{pyxb.namespace.NamespaceContext} used for this binding."""
        return self.__namespaceContext
    __namespaceContext = None

    def expandedName (self):
        """The L{expanded name<pyxb.namespace.ExpandedName>} of the
        element."""
        return self.__expandedName
    __expandedName = None

    def location (self):
        """The L{location<pyxb.utils.utility.Location>} corresponding to the
        element event."""
        return self.__location
    __location = None

    def content (self):
        """An accumulation of content to be supplied to the content model when
        the element end is reached.

        This is a list, with each member being C{(content, element_use,
        maybe_element)}.  C{content} is text or a binding instance;
        C{element_use} is C{None} or the
        L{ElementDeclaration<pyxb.binding.content.ElementDeclaration>} instance used to create
        the content; and C{maybe_element} is C{True} iff the content is
        non-content text."""
        return self.__content
    __content = None

    def __init__ (self, **kw):
        self.__expandedName = kw.get('expanded_name')
        self.__namespaceContext = kw['namespace_context']
        self.__parentState = kw.get('parent_state')
        self.__contentHandler = kw.get('content_handler')
        assert self.__contentHandler is not None
        self.__location = self.__contentHandler.location()
        self.__content = []

    def addTextContent (self, location, content):
        """Add the given text as non-element content of the current element.
        @type content: C{unicode} or C{str}
        @return: C{self}
        """
        self.__content.append(SAXInformationItem(location, content, False))

    def addElementContent (self, location, element, element_decl=None):
        """Add the given binding instance as element content corresponding to
        the given use.

        @param element: Any L{binding instance<pyxb.binding.basis._TypeBinding_mixin>}.

        @param element_decl: The L{element
        use<pyxb.binding.content.ElementDeclaration>} in the containing complex type.
        """
        self.__content.append(SAXInformationItem(location, element, True, element_decl))

class BaseSAXHandler (xml.sax.handler.ContentHandler, object):
    """A SAX handler class that maintains a stack of enclosing elements and
    manages namespace declarations.

    This is the base for L{pyxb.utils.saxdom._DOMSAXHandler} and
    L{pyxb.binding.saxer.PyXBSAXHandler}.
    """

    # An instance of L{pyxb.utils.utility.Location} that will be used to
    # construct the locations of events as they are received.
    __locationTemplate = None

    def location (self):
        """Return the current location within the SAX-processed document."""
        return self.__locationTemplate.newLocation(self.__locator)

    # The callable that creates an instance of (a subclass of)
    # L{SAXElementState} as required to hold element-specific information as
    # parsing proceeds.
    __elementStateConstructor = None

    # The namespace to use when processing a document with an absent default
    # namespace.
    def fallbackNamespace (self):
        """Return the namespace used to resolve unqualified names with no default namespace."""
        return self.__fallbackNamespace
    __fallbackNamespace = None

    # The namespace context that will be in effect at the start of the next
    # element, or C{None} if no namespace directive notifications have been
    # received since the last element start or end.  Namespace directive
    # notifications are received before the notification of element start in
    # which they apply, and cause a "next namespace context" to be allocated
    # referencing the current namespace.  The directive is applied to the next
    # context.  A non-None next context becomes active on entry to the next
    # element.  The next context is reset to None on entry to and exit from an
    # element so subsequent new directives are applied to a fresh context
    # inherited from the current context.
    __nextNamespaceContext = None

    # The namespace context that is in effect for this element.
    def namespaceContext (self):
        """Return the namespace context used for QName resolution within the
        current element.

        @return: An instance of L{pyxb.namespace.NamespaceContext}"""
        return self.__namespaceContext
    __namespaceContext = None

    # The namespace context in a schema that is including the schema to be
    # parsed by this handler.  This is necessary to handle section 4.2.1 when
    # a schema with a non-absent target namespace includes a schema with no
    # target namespace.
    __includingContext = None

    # A SAX locator object.  @todo: Figure out how to associate the
    # location information with the binding objects.
    __locator = None

    # The state for the element currently being processed
    def elementState (self):
        return self.__elementState
    __elementState = None

    # The states for all enclosing elements
    __elementStateStack = []

    def rootObject (self):
        """Return the binding object corresponding to the top-most
        element in the document

        @return: An instance of L{basis._TypeBinding_mixin} (most usually a
        L{basis.complexTypeDefinition}."""
        return self.__rootObject
    __rootObject = None

    def reset (self):
        """Reset the state of the handler in preparation for processing a new
        document.

        @return: C{self}
        """
        self.__namespaceContext = pyxb.namespace.NamespaceContext(default_namespace=self.__fallbackNamespace,
                                                                  target_namespace=self.__targetNamespace,
                                                                  including_context=self.__includingContext,
                                                                  finalize_target_namespace=False)
        self.__nextNamespaceContext = None
        self.__elementState = self.__elementStateConstructor(content_handler=self,
                                                             namespace_context=self.__namespaceContext)
        self.__elementStateStack = []
        self.__rootObject = None
        # Note: setDocumentLocator is invoked before startDocument (which
        # calls this), so this method should not reset it.
        return self

    def __init__ (self, **kw):
        """Create a new C{xml.sax.handler.ContentHandler} instance to maintain state relevant to elements.

        @keyword fallback_namespace: Optional namespace to use for unqualified
        names with no default namespace in scope.  Has no effect unless it is
        an absent namespace.

        @keyword element_state_constructor: Optional callable object that
        creates instances of L{SAXElementState} that hold element-specific
        information.  Defaults to L{SAXElementState}.

        @keyword target_namespace: Optional namespace to set as the target
        namespace.  If not provided, there is no target namespace (not even an
        absent one).  This is the appropriate situation when processing plain
        XML documents.

        @keyword location_base: An object to be recorded as the base of all
        L{pyxb.utils.utility.Location} instances associated with events and
        objects handled by the parser.
        """
        self.__includingContext = kw.pop('including_context', None)
        self.__fallbackNamespace = kw.pop('fallback_namespace', None)
        self.__elementStateConstructor = kw.pop('element_state_constructor', SAXElementState)
        self.__targetNamespace = kw.pop('target_namespace', None)
        self.__locationTemplate = pyxb.utils.utility.Location(kw.pop('location_base', None))

    def setDocumentLocator (self, locator):
        """Save the locator object."""
        self.__locator = locator

    def startDocument (self):
        """Process the start of a document.

        This resets this handler for a new document.
        @note: setDocumentLocator is invoked before startDocument
        """
        self.reset()

    def __getOrCreateNextNamespaceContext (self):
        ns_ctx = self.__nextNamespaceContext
        if ns_ctx is None:
            assert self.__namespaceContext is not None
            ns_ctx = pyxb.namespace.NamespaceContext(parent_context=self.__namespaceContext)
            self.__nextNamespaceContext = ns_ctx
        return ns_ctx

    def startPrefixMapping (self, prefix, uri):
        """Implement base class method.

        @note: For this to be invoked, the C{feature_namespaces} feature must
        be enabled in the SAX parser."""
        self.__getOrCreateNextNamespaceContext().processXMLNS(prefix, uri)

    # The NamespaceContext management does not require any action upon
    # leaving the scope of a namespace directive.
    #def endPrefixMapping (self, prefix):
    #    pass

    def startElementNS (self, name, qname, attrs):
        """Process the start of an element."""
        self.__flushPendingText()

        # Get the element name, which is already a tuple with the namespace assigned.
        expanded_name = pyxb.namespace.ExpandedName(name, fallback_namespace=self.__fallbackNamespace)

        # See if this element supports a targetNamespace attribute.  xs:schema
        # and wsdl:definitions both do.
        tns_attr = pyxb.namespace.NamespaceContext._TargetNamespaceAttribute(expanded_name)

        # If we need to assign a target namespace, we need a new context.
        # Otherwise we use the context created from pending namespace
        # directives, or we re-use the current context.
        if tns_attr is not None:
            ns_ctx = self.__getOrCreateNextNamespaceContext()
        else:
            ns_ctx = self.__nextNamespaceContext
        if ns_ctx is None:
            # Re-use the active context
            ns_ctx = self.__namespaceContext
        else:
            # Update the active context
            self.__namespaceContext = ns_ctx
        self.__nextNamespaceContext = None

        if tns_attr is not None:
            # Not true for wsdl
            #assert ns_ctx.targetNamespace() is None
            ns_ctx.finalizeTargetNamespace(attrs.get(tns_attr.uriTuple()), including_context=self.__includingContext)
            assert ns_ctx.targetNamespace() is not None

        # Save the state of the enclosing element, and create a new
        # state for this element.
        parent_state = self.__elementState
        self.__elementStateStack.append(self.__elementState)
        self.__elementState = this_state = self.__elementStateConstructor(content_handler=self,
                                                                          expanded_name=expanded_name,
                                                                          namespace_context=ns_ctx,
                                                                          parent_state=parent_state)
        return (this_state, parent_state, ns_ctx, expanded_name)

    def endElementNS (self, name, qname):
        """Process the completion of an element."""
        self.__flushPendingText()

        # Save the state of this element, and restore the state for
        # the parent to which we are returning.
        this_state = self.__elementState
        parent_state = self.__elementState = self.__elementStateStack.pop()
        # Restore namespace context and prepare for new namespace directives
        self.__namespaceContext = parent_state.namespaceContext()
        self.__nextNamespaceContext = None

        return this_state

    # We accumulate consecutive text events into a single event, primarily to
    # avoid the confusion that results when the value of a simple type is
    # represented by multiple events, as with "B &amp; W".  Also, it's faster
    # to join them all at once, and to process one content value rather than a
    # sequence of them.
    __pendingText = None
    __pendingTextLocation = None
    def __flushPendingText (self):
        if self.__pendingText:
            location = self.__pendingTextLocation
            if location is None:
                location = self.location()
            self.__elementState.addTextContent(location, ''.join(self.__pendingText))
        self.__pendingTextLocation = None
        self.__pendingText = []

    def characters (self, content):
        """Save the text as content"""
        if self.__pendingTextLocation is None:
            self.__pendingTextLocation = self.location()
        self.__pendingText.append(content)

    def ignorableWhitespace (self, whitespace):
        """Save whitespace as content too."""
        self.__pendingText.append(whitespace)

    def processingInstruction (self, target, data):
        self.__flushPendingText()

class _EntityResolver (object):
    """Dummy used to prevent the SAX parser from crashing when it sees
    processing instructions that we don't care about."""
    def resolveEntity (self, public_id, system_id):
        return io.StringIO(six.u(''))

_CreateParserModules = []
def SetCreateParserModules (create_parser_modules):
    """Provide list of modules to be used when creating parsers.

    C{xml.sax.make_parser()} takes as a parameter an optional list of modules
    which allow customization of the parser to be used.  Certain parsers have
    better support for Unicode than others.

    As an example, providing C{["drv_libxml2"]} causes the libxml2 parser to
    be used.

    The default behavior if this function is not called, or if it is called
    with an empty list or C{None}, is to provide no specific modules, which
    will result in the system default parser (probably expat).

    @param create_parser_modules: an iterable list of names of modules that
    provide a C{create_parser} function.  Pass C{None} to reset to the system
    default.  """
    global _CreateParserModules
    if create_parser_modules is None:
        _CreateParserModules = []
    else:
        _CreateParserModules = list(create_parser_modules)

def make_parser (**kw):
    """Extend C{xml.sax.make_parser} to configure the parser the way we
    need it:

      - C{feature_namespaces} is set to C{True} so we process xmlns
        directives properly
      - C{feature_namespace_prefixes} is set to C{False} so we don't get
        prefixes encoded into our names (probably redundant with the above but
        still...)

    All keywords not documented here (and C{fallback_namespace}, which is) are
    passed to the C{content_handler_constructor} if that must be invoked.

    @keyword content_handler: The content handler instance for the
    parser to use.  If not provided, an instance of C{content_handler_constructor}
    is created and used.
    @type content_handler: C{xml.sax.handler.ContentHandler}

    @keyword content_handler_constructor: A callable which produces an
    appropriate instance of (a subclass of) L{BaseSAXHandler}.  The default is
    L{BaseSAXHandler}.

    @keyword fallback_namespace: The namespace to use for lookups of
    unqualified names in absent namespaces; see
    L{pyxb.namespace.ExpandedName}.  This keyword is not used by this
    function, but is passed to the C{content_handler_constructor}.
    @type fallback_namespace: L{pyxb.namespace.Namespace}
    """
    content_handler_constructor = kw.pop('content_handler_constructor', BaseSAXHandler)
    content_handler = kw.pop('content_handler', None)
    if content_handler is None:
        content_handler = content_handler_constructor(**kw)
    parser = xml.sax.make_parser(_CreateParserModules)
    parser.setFeature(xml.sax.handler.feature_namespaces, True)
    parser.setFeature(xml.sax.handler.feature_namespace_prefixes, False)
    parser.setContentHandler(content_handler)
    # libxml2 doesn't support this feature
    try:
        parser.setEntityResolver(_EntityResolver())
    except xml.sax.SAXNotSupportedException:
        pass
    return parser

if '__main__' == __name__:
    import xml.dom.pulldom
    import xml.dom.minidom
    import pyxb.utils.saxdom as saxdom
    import time
    import lxml.sax
    import lxml.etree
    import sys

    Handler = BaseSAXHandler
    xml_file = 'examples/tmsxtvd/tmsdatadirect_sample.xml'
    if 1 < len(sys.argv):
        xml_file = sys.argv[1]
    xmld = open(xml_file, 'rb').read()

    dt1 = time.time()
    dt2 = time.time()
    dom = xml.dom.minidom.parse(io.BytesIO(xmld))
    dt3 = time.time()

    snt1 = time.time()
    saxer = make_parser(content_handler=_NoopSAXHandler())
    snt2 = time.time()
    saxer.parse(io.BytesIO(xmld))
    snt3 = time.time()

    sbt1 = time.time()
    saxer = make_parser(content_handler=BaseSAXHandler())
    sbt2 = time.time()
    saxer.parse(io.BytesIO(xmld))
    sbt3 = time.time()

    pdt1 = time.time()
    sdomer = make_parser(content_handler_constructor=saxdom._DOMSAXHandler)
    h = sdomer.getContentHandler()
    pdt2 = time.time()
    sdomer.parse(io.BytesIO(xmld))
    pdt3 = time.time()

    lst1 = time.time()
    tree = lxml.etree.fromstring(xmld)
    lst2 = time.time()
    lsh = Handler()
    lxml.sax.saxify(tree, lsh)
    lst3 = time.time()

    ldt1 = time.time()
    tree = lxml.etree.fromstring(xmld)
    ldt2 = time.time()
    ldh = xml.dom.pulldom.SAX2DOM()
    lxml.sax.saxify(tree, ldh)
    ldt3 = time.time()

    print('minidom read %f, parse %f, total %f' % (dt2-dt1, dt3-dt2, dt3-dt1))
    print('SAX+noop create %f, parse %f, total %f' % (snt2-snt1, snt3-snt2, snt3-snt1))
    print('SAX+ns create %f, parse %f, total %f' % (sbt2-sbt1, sbt3-sbt2, sbt3-sbt1))
    print('PyXB SAXDOM-based create %f, parse %f, total %f' % (pdt2-pdt1, pdt3-pdt2, pdt3-pdt1))
    print('LXML+SAX tree %f, parse %f, total %f' % (lst2-lst1, lst3-lst2, lst3-lst1))
    print('LXML+pulldom DOM tree %f, parse %f, total %f' % (ldt2-ldt1, ldt3-ldt2, ldt3-ldt1))

## Local Variables:
## fill-column:78
## End:



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-ctd-extension.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestCTDExtension (unittest.TestCase):
    def setUp (self):
        # Hide the warning about failure to convert DOM node {}third
        # to a binding
        self.__basis_log = logging.getLogger('pyxb.binding.basis')
        self.__basis_loglevel = self.__basis_log.level

    def tearDown (self):
        self.__basis_log.level = self.__basis_loglevel

    def testStructure (self):
        # Extension should be a subclass of parent
        self.assertTrue(issubclass(extendedName, personName))
        # References in subclass to parent class elements/attributes
        # should be the same, unless content model requires they be
        # different.
        self.assertEqual(extendedName.title, personName.title)

    def testPersonName (self):
        xml = '''<oldAddressee pAttr="old">
   <forename>Albert</forename>
   <forename>Arnold</forename>
   <surname>Gore</surname>
  </oldAddressee>'''
        doc = pyxb.utils.domutils.StringToDOM(xml)
        instance = oldAddressee.createFromDOM(doc.documentElement)
        self.assertEqual(2, len(instance.forename))
        # Note double dereference required because xs:anyType was used
        # as the element type
        self.assertEqual('Albert', instance.forename[0].orderedContent()[0].value)
        self.assertEqual('Arnold', instance.forename[1].orderedContent()[0].value)
        self.assertEqual('Gore', instance.surname.orderedContent()[0].value)
        self.assertEqual('old', instance.pAttr)

    def testExtendedName (self):
        xml = '''<addressee pAttr="new" eAttr="add generation">
   <forename>Albert</forename>
   <forename>Arnold</forename>
   <surname>Gore</surname>
   <generation>Jr</generation>
  </addressee>'''
        doc = pyxb.utils.domutils.StringToDOM(xml)
        instance = addressee.createFromDOM(doc.documentElement)
        self.assertEqual(2, len(instance.forename))
        self.assertEqual('Albert', instance.forename[0].orderedContent()[0].value)
        self.assertEqual('Arnold', instance.forename[1].orderedContent()[0].value)
        self.assertEqual('Gore', instance.surname.orderedContent()[0].value)
        self.assertEqual('Jr', instance.generation.orderedContent()[0].value)
        self.assertEqual('new', instance.pAttr)
        self.assertEqual('add generation', instance.eAttr)

    def testMidWildcard (self):
        # Hide the warnings that other:something could not be converted
        self.__basis_log.setLevel(logging.ERROR)
        xml = '<defs xmlns:other="other"><documentation/><other:something/><message/><message/><import/><message/></defs>'
        doc = pyxb.utils.domutils.StringToDOM(xml)
        instance = defs.createFromDOM(doc.documentElement)
        self.assertFalse(instance.documentation is None)
        self.assertEqual(3, len(instance.message))
        self.assertEqual(1, len(instance.import_))
        self.assertEqual(1, len(instance.wildcardElements()))

        xml = '<defs xmlns:other="other"><other:something/><other:else/><message/><message/><import/><message/></defs>'
        doc = pyxb.utils.domutils.StringToDOM(xml)
        instance = defs.createFromDOM(doc.documentElement)
        self.assertTrue(instance.documentation is None)
        self.assertEqual(3, len(instance.message))
        self.assertEqual(1, len(instance.import_))
        self.assertEqual(2, len(instance.wildcardElements()))

    def testEndWildcard (self):
        # Hide the warnings that other:something could not be converted
        self.__basis_log.setLevel(logging.ERROR)
        xml = '<defs xmlns:other="other"><message/><other:something/></defs>'
        doc = pyxb.utils.domutils.StringToDOM(xml)
        self.assertRaises(UnrecognizedContentError, defs.createFromDOM, doc.documentElement)

if __name__ == '__main__':
    unittest.main()
